#! /usr/bin/python
#
#  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
#  USA.

import sys
import string
import os
import stat
import tempfile
import shutil
import md5
import getopt
import glob
import time
import types

#
# Globals
#

PACTEST_NAME    = "PACTEST"
PACTEST_VERSION = "0.1"

# ALPM
PM_ROOT     = "/"
PM_DBPATH   = "var/lib/pacman"
PM_CACHEDIR = "var/cache/pacman/pkg"
PM_EXT_PKG  = ".pkg.tar.gz"
PM_EXT_DB   = ".db.tar.gz"
# Pacman
PACCONF     = "etc/pacman.conf"
# Pactest specific
TMPDIR      = "tmp"
SYNCREPO    = "var/pub"
LOGFILE     = "var/log/pactest.log"

#
# Methods
#

def usage():
	print

def error(msg):
	print "error: " + msg
	sys.exit(1)

def vprint(msg):
	if verbose:
		print msg

def vvprint(msg):
	if verbose > 1:
		print msg

def mkfile(name, data = ""):
	filename = name
	path = os.path.dirname(filename)
	try:
		if path and not os.path.isdir(path):
			os.makedirs(path, 0755)
	except:
		error("mkfile: could not create directory hierarchy '" + path + "'")

	f = file(filename, "w")
	f.write(data)
	f.close()

def mkinstallfile(filename):
	data = ""
	for i in ["pre_install", "post_install",
	          "pre_upgrade", "post_upgrade",
	          "pre_remove", "post_remove"]:
		data += i + "() {\n" \
		          + "\techo :: scriplet:" + i + "\n" \
		          + "}\n"
	mkfile(filename, data + "\n")

def mkcfgfile(filename, option, syncdbs):
	# Options
	data = "[options]\n"
	for i in option['noupgrade']:
		data += "noupgrade = " + i + "\n"
	for i in option['ignorepkg']:
		data += "ignorepkg = " + i + "\n"
	for i in option['noextract']:
		data += "noextract = " + i + "\n"
	# Repositories
	for i in syncdbs:
		data += "[" + i.treename + "]\n" \
		        + "server = file://" + SYNCREPO + "/" + i.treename + "\n"

	mkfile(filename, data + "\n")

def readsection(f):
	i = []
	while 1:
		line = f.readline().strip("\n")
		if not line:
			break
		i.append(line)
	return i

def mksection(title, data):
	s = "%" + title + "%\n"
	if type(data) == type([]):
		for i in data:
			s = s + i + "\n"
	else:
		s = s + str(data) + "\n"
	return s + "\n"

def mkfilelist(files):
	i = []
	for f in files:
		i.append(f)
		dir = f
		while "/" in dir:
			[dir, tmp] = dir.rsplit("/", 1)
			if not dir + "/" in files:
				i.append(dir + "/")
	i.sort()
	return i

def getmd5sum(filename):
	f = open(filename, "rb")
	sum = md5.new()
	while 1:
		block = f.read(1048576)
		if not block:
			break
		sum.update(block)
	f.close()
	d = sum.digest()
	return "%02x"*len(d) % tuple(map(ord, d))

def getmtime(filename):
	st = os.stat(filename)
	return st[stat.ST_ATIME], st[stat.ST_MTIME], st[stat.ST_CTIME]

#
# File object
#

class pmfile:
	def __init__(self, root, name):
		self.name = name
		self.root = root

		filename = os.path.join(self.root, self.name)
		self.checksum = getmd5sum(filename)
		self.mtime = getmtime(filename)

	def dump(self):
		print "\t\t" + self.name, "(" + str(self.checksum) + "/" + str(self.mtime) + ")"

#
# Package object
#

class pmpkg:
	def __init__(self, name, version = "1.0-1"):
		# desc
		self.name = name
		self.version = version
		self.desc = ""
		self.groups = []
		self.url = ""
		self.license = []
		self.arch = ""
		self.builddate = ""
		self.installdate = ""
		self.packager = ""
		self.size = 0
		self.csize = 0
		self.reason = 0
		self.md5sum = ""      # sync only
		self.replaces = []    # sync only (will be moved to depends)
		self.force = 0        # sync only (will be moved to depends)
		# depends
		self.depends = []
		self.requiredby = []  # local only
		self.conflicts = []
		self.provides = []
		# files
		self.files = []
		self.backup = []
		# install
		self.install = 0

		self.checksum = {
			'desc': "",
			'depends': "",
			'files': "",
			'install': ""
		}
		self.mtime = {
			'desc': (0, 0, 0),
			'depends': (0, 0, 0),
			'files': (0, 0, 0),
			'install': (0, 0, 0)
		}

	def fullname(self):
		return self.name + "-" + self.version

	def install_files(self, root):
		for f in self.files:
			mkfile(os.path.join(root, f), f + "\n")

	def makepkg(self, path):
		archive = os.path.join(path, self.fullname() + PM_EXT_PKG)

		curdir = os.getcwd()
		tmpdir = tempfile.mkdtemp()
		os.chdir(tmpdir)

		# Generate package file system
		for f in self.files:
			mkfile(f, f + "\n")
			self.size = self.size + os.stat(f)[stat.ST_SIZE]

		# .PKGINFO
		data = "pkgname = " + self.name + "\n"
		data += "pkgver = " + self.version + "\n"
		data += "pkgdesc = " + self.desc + "\n"
		data += "url = " + self.url +"\n"
		data += "builddate = " + self.builddate + "\n"
		data += "packager = " + self.packager + "\n"
		data += "size = " + str(self.size) + "\n"
		if self.arch:
			data += "arch = " + self.arch + "\n"
		for i in self.license:
			data += "license = " + i + "\n"
		for i in self.replaces:
			data += "replaces = " + i + "\n"
		for i in self.groups:
			data += "group = " + i + "\n"
		for i in self.depends:
			data += "depend = " + i + "\n"
		for i in self.conflicts:
			data += "conflict = " + i + "\n"
		for i in self.provides:
			data += "provides = " + i + "\n"
		for i in self.backup:
			data += "backup = " + i + "\n"
		mkfile(".PKGINFO", data)
		targets = ".PKGINFO"

		# .INSTALL
		if self.install:
			data = ""
			mkinstallfile(".INSTALL")
			targets = targets + " .INSTALL"

		# .FILELIST
		if self.files:
			os.system("tar cvf /dev/null * | sort >.FILELIST")
			targets = targets + " .FILELIST *"

		# Generate package archive
		os.system("tar zcf " + archive + " " + targets)

		os.chdir(curdir)
		shutil.rmtree(tmpdir)

	def dump(self):
		print "\t\t[", self.fullname(), "]"
		print "\t\tdepends:", list2str(self.depends)
		print "\t\tfiles:", list2str(self.files)

#
# Database object
#

class pmdb:
	def __init__(self, treename, dbdir):
		self.treename = treename
		self.dbdir = dbdir
		self.pkgs = []

	def getpkg(self, name):
		for pkg in self.pkgs:
			if name == pkg.name:
				return pkg

	def db_read(self, name):
		path = os.path.join(self.dbdir, self.treename)
		if not os.path.isdir(path):
			return

		dbentry = ""
		for roots, dirs, files in os.walk(path):
			for i in dirs:
				[pkgname, pkgver, pkgrel] = i.rsplit("-", 2)
				if pkgname == name:
					dbentry = i
					break
		if not dbentry:
			return None
		path = os.path.join(path, dbentry)

		[pkgname, pkgver, pkgrel] = dbentry.rsplit("-", 2)
		pkg = pmpkg(pkgname, pkgver + "-" + pkgrel)

		# desc
		filename = os.path.join(path, "desc")
		fd = file(filename, "r")
		while 1:
			line = fd.readline()
			if not line:
				break
			line = line.strip("\n")
			if line == "%NAME%":
				pkg.name = fd.readline().strip("\n")
			elif line == "%VERSION%":
				pkg.version = fd.readline().strip("\n")
			elif line == "%GROUPS%":
				pkg.groups = readsection(fd)
			elif line == "%REASON%":
				pkg.reason = int(fd.readline().strip("\n"))
			elif line == "%SIZE%" or line == "%CSIZE%":
				pkg.size = int(fd.readline().strip("\n"))
		fd.close()
		pkg.checksum['desc'] = getmd5sum(filename)
		pkg.mtime['desc'] = getmtime(filename)

		# depends
		filename = os.path.join(path, "depends")
		fd = file(filename, "r")
		while 1:
			line = fd.readline()
			if not line:
				break
			line = line.strip("\n")
			if line == "%DEPENDS%":
				pkg.depends = readsection(fd)
			elif line == "%REQUIREDBY%":
				pkg.requiredby = readsection(fd)
			elif line == "%CONFLICTS%":
				pkg.conflicts = readsection(fd)
			elif line == "%PROVIDES%":
				pkg.provides = readsection(fd)
		fd.close()
		pkg.checksum['depends'] = getmd5sum(filename)
		pkg.mtime['depends'] = getmtime(filename)

		# files
		filename = os.path.join(path, "files")
		fd = file(filename, "r")
		while 1:
			line = fd.readline()
			if not line:
				break
			line = line.strip("\n")
			if line == "%FILES%":
				while line:
					line = fd.readline().strip("\n")
					if line and line[-1] != "/":
						pkg.files.append(line)
			if line == "%BACKUP%":
				pkg.backup = readsection(fd)
		fd.close()
		pkg.checksum['files'] = getmd5sum(filename)
		pkg.mtime['files'] = getmtime(filename)

		# install
		filename = os.path.join(path, "install")
		if os.path.isfile(filename):
			pkg.checksum['install'] = getmd5sum(filename)
			pkg.mtime['install'] = getmtime(filename)

		return pkg

	#
	# db_write is used to add both 'local' and 'sync' db entries
	#
	def db_write(self, pkg):
		path = os.path.join(self.dbdir, self.treename, pkg.fullname())
		if not os.path.isdir(path):
			os.makedirs(path);

		# desc
		# for local db entries: name, version, desc, groups, url, license,
		#                       arch, builddate, installdate, packager,
		#                       size, reason
		# for sync entries: name, version, desc, groups, csize, md5sum,
		#                   replaces, force
		data = mksection("NAME", pkg.name)
		data += mksection("VERSION", pkg.version)
		if pkg.desc:
			data += mksection("DESC", pkg.desc)
		if pkg.groups:
			data += mksection("GROUPS", pkg.groups)
		if self.treename == "local":
			if pkg.url:
				data += mksection("URL", pkg.url)
			if pkg.license:
				data += mksection("LICENSE", pkg.license)
			if pkg.arch:
				data += mksection("ARCH", pkg.arch)
			if pkg.builddate:
				data += mksection("BUILDDATE", pkg.builddate)
			if pkg.installdate:
				data += mksection("INSTALLDATE", pkg.installdate)
			if pkg.packager:
				data += mksection("PACKAGER", pkg.packager)
			if pkg.size:
				data += mksection("SIZE", pkg.size)
			if pkg.reason:
				data += mksection("REASON", pkg.reason)
		else:
			if pkg.csize:
				data += mksection("CSIZE", pkg.csize)
			if pkg.md5sum:
				data += mksection("MD5SUM", pkg.md5sum)
			if pkg.replaces:
				data += mksection("REPLACES", pkg.replaces)
			if pkg.force:
				data += "%FORCE%\n\n"
		filename = os.path.join(path, "desc")
		mkfile(filename, data)
		pkg.checksum['desc'] = getmd5sum(filename)
		pkg.mtime['desc'] = getmtime(filename)

		# depends
		# for local db entries: depends, requiredby, conflicts, provides
		# for sync ones: depends, conflicts, provides
		data= ""
		if pkg.depends:
			data += mksection("DEPENDS", pkg.depends)
		if self.treename == "local":
			if pkg.requiredby:
				data += mksection("REQUIREDBY", pkg.requiredby)
		if pkg.conflicts:
			data += mksection("CONFLICTS", pkg.conflicts)
		if pkg.provides:
			data += mksection("PROVIDES", pkg.provides)
		filename = os.path.join(path, "depends")
		mkfile(filename, data)
		pkg.checksum['depends'] = getmd5sum(filename)
		pkg.mtime['depends'] = getmtime(filename)

		# files
		# for local entries, fields are: files, backup
		# for sync ones: none
		if self.treename == "local":
			data = ""
			if pkg.files:
				data += mksection("FILES", mkfilelist(pkg.files))
			if pkg.backup:
				data += "%BACKUP%\n"
				for i in pkg.backup:
					# ORE: TBD
					data += i + "\t" + "XXXXXXXXXX" + "\n"
				data += "\n"
			filename = os.path.join(path, "files")
			mkfile(filename, data)
			pkg.checksum['files'] = getmd5sum(filename)
			pkg.mtime['files'] = getmtime(filename)

		# install
		if self.treename == "local" and pkg.install:
			filename = os.path.join(path, "install")
			mkfile(filename, mkinstallfile())
			pkg.checksum['install'] = getmd5sum(filename)
			pkg.mtime['install'] = getmtime(filename)

	# ORE: TBD - is it really useful?
	def gensync(self, path):
		archive = os.path.join(path, self.treename, PM_EXT_DB)
		dbpath = os.path.join(self.dbdir, self.treename)
		#os.system("tar cf " + archive + " -C " + dbpath + " *")

	def dump(self):
		print "\t[", self.treename, "]"
		print "\tpkgs:"
		for pkg in self.pkgs:
			pkg.dump()

#
# Rule object
#

class pmrule:
	def __init__(self, rule):
		self.rule = rule
		self.false = 0
		self.result = 0

	def check(self, root, retcode, localdb, files):
		success = 1

		[test, args] = self.rule.split("=")
		if test[0] == "!":
			self.false = 1
			test = test[1:]
		[type, case] = test.split("_")
		if "|" in args:
			[key, item] = string.split(args, "|", 1)
		else:
			[key, item] = [args, None]

		if type == "PACMAN":
			if case == "RETCODE":
				if retcode != int(key):
					success = 0
			elif case == "OUTPUT":
				found = 0
				fd = file(os.path.join(root, LOGFILE), "r")
				while 1:
					line = fd.readline()
					if not line:
						break
					if line.find(key) != -1:
						found = 1
				fd.close()
				if found == 0:
					success = 0
			else:
				success = -1
		elif type == "PKG":
			newpkg = localdb.db_read(key)
			if verbose > 1 and newpkg:
				print "newpkg.checksum:", newpkg.checksum
				print "newpkg.mtime:", newpkg.mtime
			oldpkg = localdb.getpkg(key)
			if oldpkg and verbose > 1 and oldpkg:
				print "oldpkg.checksum:", oldpkg.checksum
				print "oldpkg.mtime:", oldpkg.mtime
			if not newpkg:
				success = 0
			else:
				if case == "EXIST":
					success = 1
				elif case == "MODIFIED":
					if item:
						if (oldpkg.mtime[item][1], oldpkg.mtime[item][2]) \
						    == (newpkg.mtime[item][1], newpkg.mtime[item][2]):
							success = 0
					else:
						for i in ["desc", "depends", "files", "install"]:
							if i == "install" \
							   and oldpkg.mtime[i] == (0, 0, 0) \
							   and newpkg.mtime[i] == (0, 0, 0):
								continue
							if (oldpkg.mtime[i][1], oldpkg.mtime[i][2]) \
							    == (newpkg.mtime[i][1], newpkg.mtime[i][2]):
								success = 0
				elif case == "VERSION":
					if item != newpkg.version:
						success = 0
				elif case == "GROUPS":
					if not item in newpkg.groups:
						success = 0
				elif case == "DEPENDS":
					if not item in newpkg.depends:
						success = 0
				elif case == "REQUIREDBY":
					if not item in newpkg.requiredby:
						success = 0
				elif case == "REASON":
					if not newpkg.reason == int(item):
						success = 0
				elif case == "FILES":
					if not item in newpkg.files:
						success = 0
				elif case == "BACKUP":
					success = -1
				else:
					success = -1
		elif type == "FILE":
			filename = os.path.join(root, key)
			if case == "EXIST":
				if not os.path.isfile(filename):
					success = 0
			else:
				if case == "MODIFIED":
					for f in files:
						if f.name == args:
							checksum = getmd5sum(filename)
							mtime = getmtime(filename)
							if verbose > 1:
								vvprint("\tnew (" + checksum + "/" + str(mtime) + ")")
								vvprint("\told (" + f.checksum + "/" + str(f.mtime) + ")")
							if (f.mtime[1], f.mtime[2]) == (mtime[1], mtime[2]):
								success = 0
				elif case == "PACNEW":
					if not os.path.isfile(filename + ".pacnew"):
						success = 0
				elif case == "PACORIG":
					if not os.path.isfile(filename + ".pacorig"):
						success = 0
				elif case == "PACSAVE":
					if not os.path.isfile(filename + ".pacsave"):
						success = 0
				else:
					success = -1
		else:
			success = -1

		if self.false and success != -1:
			success = not success

		self.result = success

		return success

#
# Test object
#

class pmtest:
	def __init__(self, name, root):
		self.name = name
		self.root = root

	def addpkg2db(self, treename, pkg):
		if treename == "local":
			self.localdb.pkgs.append(pkg)
		else:
			found = 0
			for sdb in self.syncdbs:
				if sdb.treename == treename:
					sdb.pkgs.append(pkg)
					found = 1
			if not found:
				sdb = pmdb(treename,
				           os.path.join(self.root, PM_DBPATH))
				sdb.pkgs.append(pkg)
				self.syncdbs.append(sdb)

	def addpkg(self, pkg):
		self.localpkgs.append(pkg)

	def addrule(self, rulename):
		rule = pmrule(rulename)
		self.rules.append(rule)

	def load(self):
		# Reset test parameters
		self.result = {
			'passed': 0,
			'failed': 0
		}
		self.args = ""
		self.retcode = 0
		self.localdb = pmdb("local",
		                    os.path.join(self.root, PM_DBPATH))
		self.syncdbs = []
		self.localpkgs = []
		self.filesystem = []

		self.description = ""
		self.option = {
			'noupgrade': [],
			'ignorepkg': [],
			'noextract': []
		}

		# Test rules
		self.rules = []
		self.files = []
		
		if os.path.isfile(self.name):
			execfile(self.name)
		else:
			error("file " + self.name + " does not exist!")

	def generate(self):
		vprint("==> Generating test environment")

		# Cleanup leftover files from a previous test session
		if os.path.isdir(self.root):
			shutil.rmtree(self.root)

		# Create directory structure
		vprint("* Creating directory structure:")
		dbdir = os.path.join(self.root, PM_DBPATH)
		cachedir = os.path.join(self.root, PM_CACHEDIR)
		syncdir = os.path.join(self.root, SYNCREPO)
		tmpdir = os.path.join(self.root, TMPDIR)
		logdir = os.path.join(self.root, os.path.dirname(LOGFILE))
		for i in [dbdir, cachedir, syncdir, tmpdir, logdir]:
			if not os.path.isdir(i):
				os.makedirs(i, 0755)

		# Configuration file
		vprint("* Creating configuration file")
		mkcfgfile(os.path.join(self.root, PACCONF), self.option, self.syncdbs)

		# Creating packages
		vprint("* Creating package archives")
		for pkg in self.localpkgs:
			vprint("\t" + os.path.join(TMPDIR,
			                           pkg.fullname() + PM_EXT_PKG))
			pkg.makepkg(tmpdir)
		for db in self.syncdbs:
			for pkg in db.pkgs:
				archive = pkg.fullname() + PM_EXT_PKG
				vprint("\t" + os.path.join(PM_CACHEDIR, archive))
				pkg.makepkg(cachedir)
				pkg.md5sum = getmd5sum(os.path.join(cachedir, archive))
				pkg.csize = os.stat(os.path.join(cachedir, archive))[stat.ST_SIZE]

		# Populating databases
		vprint("* Populating databases")
		for pkg in self.localdb.pkgs:
			pkg.installdate = time.ctime()
			self.localdb.db_write(pkg)
		for db in self.syncdbs:
			for pkg in db.pkgs:
				db.db_write(pkg)

		# Creating sync database archives
		vprint("* Creating sync database archives")
		for db in self.syncdbs:
			archive = db.treename + PM_EXT_DB
			vprint("\t" + os.path.join(SYNCREPO, archive))
			db.gensync(syncdir)

		# Local package archives
		vprint("* Creating local package archives")

		# Filesystem
		vprint("* Populating file system")
		for pkg in self.localdb.pkgs:
			pkg.install_files(self.root)
		for f in self.filesystem:
			vprint("\t" + f)
			mkfile(os.path.join(self.root, f), f + "\n")

		# Done.
		vprint("* Taking a snapshot of the file system")
		for roots, dirs, files in os.walk(self.root):
			for i in files:
				filename = os.path.join(roots, i)
				f = pmfile(self.root, filename.replace(self.root + "/", ""))
				self.files.append(f)
				vvprint("\t" + f.name)

	def run(self, pacman):
		vprint("==> Running 'pacman " + self.args +"'")
		cmd = "fakeroot " + pacman['bin'] + " "
		cmd += "--noconfirm "
		cmd += "--config=" + os.path.join(self.root, PACCONF) + " "
		cmd += "--root=" + self.root + " "
		if pacman['debug']:
			cmd += "--debug=" + str(pacman['debug']) + " "
		cmd += self.args + " "
		cmd += ">" + os.path.join(self.root, LOGFILE) + " 2>&1"

		# Change to the tmp dir before running pacman, so that local package
		# archives are made available more easily.
		curdir = os.getcwd()
		tmpdir = os.path.join(self.root, TMPDIR)
		os.chdir(tmpdir)
		self.retcode = os.system(cmd)
		if self.retcode == None:
			self.retcode = 0
		else:
			self.retcode /= 256
		vvprint("\tretcode = " + str(self.retcode))
		os.chdir(curdir)

	def check(self):
		vprint("==> Testing results")

		for i in self.rules:
			success = i.check(self.root, self.retcode, self.localdb, self.files)
			if success == 1:
				msg = "OK"
				self.result['passed'] += 1
			elif success == 0:
				msg = "KO"
				self.result['failed'] += 1
			else:
				msg = "SKIP"
			print "[" + msg + "] " + i.rule
			i.result = success

#
# Environment object
#

class pmenv:
	def __init__(self, root = "root"):
		self.root = os.path.abspath(root)
		self.pacman = {
			'bin': "pacman",
			'debug': 0
		}
		self.testcases = []

	def addtest(self, testname):
		if not os.path.isfile(testname):
			return
		test = pmtest(testname, self.root)
		self.testcases.append(test)

	def run(self):
		if not self.testcases:
			return

		for t in self.testcases:
			print "==========" * 8
			print "Running '" + t.name + "'"

			t.load()

			print t.description
			print "----------" * 8

			t.generate()

			# Hack for mtimes consistency
			modified = 0
			for i in t.rules:
				if i.rule.find("MODIFIED") != -1:
					modified = 1
			if modified:
				time.sleep(2)

			t.run(self.pacman)
			t.check()

			print "==>",
			if t.result['failed'] == 0:
				print "SUCCESS"
			else:
				print "FAILURE"
			print

	def results(self):
		if not self.testcases:
			return

		passed = 0
		print "==========" * 8
		print "Results"
		print "----------" * 8
		for test in self.testcases:
			ok = test.result['passed']
			ko = test.result['failed']
			rules = len(test.rules)
			if test.result['failed'] == 0:
				print "[SUCCESS]",
				passed += 1
			else:
				print "[FAILURE]",
			print test.name,
			print "\tRules:",
			print "OK=" + str(ok) + "/" + str(rules),
			print "KO=" + str(ko) + "/" + str(rules),
			print "SKIP=" + str(rules-(ok+ko)) + "/" + str(rules)
		print "----------" * 8
		total = len(self.testcases)
		failed = total - passed
		print "PASSED=" + str(passed) + "/" + str(total), "(" + str(passed*100/total) + "%)",
		print "FAILED=" + str(failed) + "/" + str(total), "(" + str(failed*100/total) + "%)"
		print

#
# Main
#

env = pmenv()

testcases = ""

verbose = 0
debug = 0

opts, args = getopt.getopt(sys.argv[1:],
                           "d:hp:t:v",
                           ["debug=", "help", "pacman=", "test=", "verbose"])
for (cmd, param) in opts:
	if cmd == "-v" or cmd == "--verbose":
		verbose += 1
	elif cmd == "-d" or cmd == "--debug":
		env.pacman['debug'] = int(param)
	elif cmd == "-t" or cmd == "--test":
		testcases = glob.glob(param)
	elif cmd == "-p" or cmd == "--pacman":
		env.pacman['bin'] = param

for i in testcases:
	env.addtest(i)

env.run()
env.results()
